ifndef bipedPhy_wdl;
define bipedPhy_wdl;



// ---------------------------------------------------------------------
// STARTHEADER
//
// ver: 5.0
// engineReq: 6.4
// date: 061011
//
// title: Biped Physics (01)
// class: MOVEMENT
// type: SUPPORT
// image: bipedPhy01.pcx
// help: Handle the physics of motion for bipeds in the world.
//
// idcode: 033
// prefix: bipedPhy01_
//
// ENDHEADER
// ---------------------------------------------------------------------



// =====================================================================
// USER DEFINED VALUES

// section: Friction

// entry: "Glide"
// help: Friction for gliding along surfaces (0..1)
// id: 10
var	move_friction = 0.25;              // engine var used by c_move

// entry: Ground
// help: Friction while on solid ground
// id: 11
var	bipedPhy01_grnd_friction = 0.5;

// entry: Ground Angular
// help: Effects panning on ground
// id: 12
var	bipedPhy01_grnd_ang_fric = 1.0;        //---

// entry: Air
// help: Friction used when jumping/falling.
// id: 13
var	bipedPhy01_air_friction = 0.1;

// entry: Water
// help: Friction used while swimming
// id: 14
var	bipedPhy01_water_friction = 1.65;
// entry: Water Angular
// help: Effects panning in water
// id: 15
var	bipedPhy01_water_ang_friction = 1.65;


//---------------------------------------------------
// section: Environmental Forces

// enable: Use Passable as Water?
// help: If set, bipeds treat passable blocks as 'water'
// id: 23
define	bipedPhy01_passable_water;


// entry: Float force
// help: this is the amount of upward force applied to bipeds in a passible surface.
// id: 16
var	bipedPhy01_water_float = 1.85;


// entry: Gravity
// help: Downward pull on bipeds (keeps them from floating away)
// id: 17
var	bipedPhy01_gravity = 9;




//----------------------------------------------------------------------
// section: Damage from Falling

// entry: Safe Time
// help: Maximum time biped can fall without damage (ticks)
// id: 19
define bipedPhy01_safe_fall_time_const = 9;

// entry: Damage Const
// help: Constant used to calculate damage from falling
// id: 20
define bipedPhy01_fall_damage_const = 1.75;

//----------------------------------------------------------------------
// section: Damage from drowning

// entry: Safe Time
// help: Time underwater before taking damage (ticks)
// id: 21
define	bipedPhy01_safe_underwater_time_const = 220;

// entry: Damage
// help: Damage per tick while drowning
// id: 22
define	bipedPhy01_drowning_damage_const = 0.125;


///////////////////////////////////////////////////////////////////////
// Entity skill & flag definitions

define	_health__003,SKILL1;	// health of the biped
define	_armor__003, SKILL2;	// armor protects from some sorts of damage


// max force vectors
define _max_force_x__003,SKILL3;
define _max_force_y__003,SKILL4;
define _max_ang_force_pan__003,SKILL6;
//DEFINE _biped01_max_ang_force_tilt,SKILL7;
//DEFINE _biped01_max_ang_force_roll,SKILL8;

define _trigger_range__003, SKILL7;    // distace that things are triggered

define _force_mult_003,SKILL8;

// height limits
//define _step_height__003,SKILL9;   // REPLACED with _move_min_z__003
define _move_min_z__003,SKILL9;       // used to set the move_min_z durring moves
define _jump_height__003,SKILL10;
define _duck_height__003,SKILL11;





// velocity vectors
/*
// Velocity in quants/tick
define	_gid01_vel_x, skill21;        // current velocity (quants/tick)
define	_gid01_vel_y, skill22;        // current velocity (quants/tick)
define	_gid01_vel_z, skill23;        // current velocity (quants/tick)


// ang velocity in deg/tick
define _gid01_ang_vel_pan, SKILL24;
define _gid01_ang_vel_tilt, SKILL25;
define _gid01_ang_vel_roll, SKILL26;
*/
define _biped01_dist, SKILL27;  // distance covered last frame

define _biped01_state, skill28;	// biped state:
define biped01_state_none_const,0;		// no movement (i.e. player dead)
//--define biped01_state_stand_const,1;		// standing
define biped01_state_walk_const,2;    	// walking. running, standing
//--define biped01_state_run_const,3;     	// running
define biped01_state_jump_const,4;    	// jumping
define biped01_state_duck_const,5;    	// ducking or crawling
define biped01_state_crawl_const,6;		// crawling (move while ducked)
define biped01_state_swim_const,7;		// swimming
define biped01_state_wade_const,8;		// wading in water
define biped01_state_dying_const,10; 	// dying
define biped01_state_dead_const,11;    // finished dying
define biped01_state_activate_const,12;// press a button or switch
define biped01_state_override_const,13;  	// bipedPhy is being overriden at the movement with something else (driving, climbing, etc.)


define _biped01_state_mod, skill29;	// biped state modifier:
define biped01_state_mod_none_const,0;					// no modifier
define biped01_state_mod_fall_const,1;			// biped is falling
define biped01_state_mod_underwater_const,2;	// biped is underwater
define biped01_state_mod_damage_const,3;		// biped has taken damage from environment (falling, crushing, etc.)
define biped01_state_mod_drown_const,4;		// biped is drowning
define _biped01_state_mod_value, skill30;	// state modifier value, dependent on state modifier

// note: state 2 modifiers can be used in combination with state 1 modifier above
define _biped01_state2_mod, skill34; // biped state modifier 2:
define biped01_state2_mod_none_const,0;	// no modifier
define biped01_state2_mod_attack_const,1;	// preforming an attack
define biped01_state2_mod_damage_const,2;	// taking damage
define _biped01_state2_mod_value, skill35;	// state2 modifier value, dependent on state2 modifier


define _biped01_jump_target, SKILL32;	// target to reach while jumping

define _biped01_anim_dist, SKILL33;    // animation distance



define _no_fall_dam__003, FLAG1;			// if set, entity does not take damage from falls



//----------------------------------------------------------------------
// "Local" global values (used for parameter passing)
var	bipedPhy01_height;						// height from the bottom of the model to the nearest solid surface.
//var	bipedPhy01_passable_height;			// height from the bottom of the model to the nearest passable surface. (0 if none)
var	bipedPhy01_surface_normal_vec[3];	// surface normal of the surface beneight the ent
var	bipedPhy01_surface_vel_vec[3];		// the velocity of the surface under the ent

define	bipedPhy01_scan_dist_const = 12;	// max distance we scan under our biped.

entity* bipedPhy01_temp_ent;					// entity under biped
entity* bipedPhy01_content_ent;				// entity biped is in



// enable: Debug?
// help: Display debug information?
//define	BIPEDPHY01_DEBUG;


ifdef	BIPEDPHY01_DEBUG;
var bipedPhy01_debug_data[20];

font bipedPhy01_debug_font = <ackfont.pcx>,6,9;
text bipedPhy01_debug_txt
{
     font = bipedPhy01_debug_font;
     pos_x = 430;
     pos_y = 1;
     string = "Scan Results (C_Collide)\n============\n->height(*1000)\n->trace_hit\n->state\n#3\n#4\n#5\n->#6(x100)\n->#7(x100)\n->#8(x100)\n->#9\n->#10\n->#11\n->#12\n->dist_x(x100)\n->dist_y(x100)\n->dist_z(x100)\n->abs_x(x100)\n->abs_y(x100)\n->abs_z(x100)\n->move_min_z(x100)";
     flags = d3d, visible;
}


panel bipedPhy01_debug_pan
{
     pos_x = 400;
     pos_y = 10;
     digits = 0, 9,  5, bipedPhy01_debug_font, 1000, bipedPhy01_debug_data[0];  //   height
     digits = 0, 18, 5, bipedPhy01_debug_font, 1, bipedPhy01_debug_data[1];			// trace_hit
     digits = 0, 27, 5, bipedPhy01_debug_font, 1, bipedPhy01_debug_data[2]; 		// state
     digits = 0, 36, 5, bipedPhy01_debug_font, 1, bipedPhy01_debug_data[3]; //
     digits = 0, 45, 5, bipedPhy01_debug_font, 1, bipedPhy01_debug_data[4]; //

     digits = 0, 63, 5, bipedPhy01_debug_font, 100, bipedPhy01_debug_data[5]; // normal.x
     digits = 0, 72, 5, bipedPhy01_debug_font, 100, bipedPhy01_debug_data[6]; // normal.y
     digits = 0, 81, 5, bipedPhy01_debug_font, 100, bipedPhy01_debug_data[7]; // normal.z
     digits = 0, 90, 5, bipedPhy01_debug_font, 100, bipedPhy01_debug_data[8]; // you
     digits = 0, 99, 5, bipedPhy01_debug_font, 100, bipedPhy01_debug_data[9]; //
     digits = 0, 108, 5, bipedPhy01_debug_font, 100, bipedPhy01_debug_data[10]; //dist
     digits = 0, 117, 5, bipedPhy01_debug_font, 100, bipedPhy01_debug_data[11];   // dist / time
     digits = 0, 126, 5, bipedPhy01_debug_font, 100, bipedPhy01_debug_data[12];
     digits = 0, 135, 5, bipedPhy01_debug_font, 100, bipedPhy01_debug_data[13];
     digits = 0, 144, 5, bipedPhy01_debug_font, 100, bipedPhy01_debug_data[14];
     digits = 0, 153, 5, bipedPhy01_debug_font, 100, bipedPhy01_debug_data[15];
     digits = 0, 162, 5, bipedPhy01_debug_font, 100, bipedPhy01_debug_data[16];
     digits = 0, 171, 5, bipedPhy01_debug_font, 100, bipedPhy01_debug_data[17];
     digits = 0, 180, 5, bipedPhy01_debug_font, 100, bipedPhy01_debug_data[18];
     digits = 0, 189, 5, bipedPhy01_debug_font, 100, bipedPhy01_debug_data[19];
     flags =  d3d, visible;
}
endif;

// =====================================================================
// MISC FUNCTIONS

// Calculate the damage from a fall and return that value
function	BipedPhy01_Fall_Damage()
{
	if(my._no_fall_dam__003 == 1) { return(0); }

	// calculate damage depending on fall time
 	return(10 + INT(((my._biped01_state_mod_value - bipedPhy01_safe_fall_time_const) * bipedPhy01_fall_damage_const)));
}



// =====================================================================
// SETUP FUNCTIONS


// Desc: Setup an entity so it can use BipedPhy01_Update
//
// NOTE: make sure the entity has been loaded at least one frame before calling (wait(1)).
function BipedPhy01_Setup_Ent()
{
	my.fat = ON; my.narrow = ON;
	c_setminmax(my);
	// make symmetrical in X/Y
	temp.x = max(-my.min_x,my.max_x);
	temp.y = max(-my.min_y,my.max_y);
	temp.z = max(temp.x,temp.y);
	my.min_x = -temp.z; my.max_x = temp.z;
	my.min_y = -temp.z; my.max_y = temp.z;

}


// =====================================================================
// C_COLLIDE CODE

// =====================================================================
// ROTATION (PAN) FUNCTIONS

// Desc: handle biped panning (old, not used anymore)
//
// Input: ang_force (vec) set to the panning force
//	       ang_fric is set to whatever friction
// Result:
//		Sets: my._gid01_ang_vel_pan equal to the angular velocity
// Assume: my is set to a biped01 entity
function BipedPhy01_Pan_Force(pan_force,ang_fric)
{
	var	delta_ang_fric;

	delta_ang_fric = max((1-ang_fric * time_step),0);

	my._gid01_ang_vel_pan = (time_step * pan_force) + (delta_ang_fric * my._gid01_ang_vel_pan);

//	temp.pan += my._gid01_ang_vel_pan * time_step;
	move_mode = ignore_passable + ignore_passents + ignore_push + activate_trigger + glide + use_box;
	c_rotate(my,vector((my._gid01_ang_vel_pan * time_step),0,0),move_mode);

}


// Desc: handle biped panning with absolute values (not time correct)
//
// Input: ang_dist (vec) set to the panning amount (modified)
//
// Result:
//		Sets: my._gid01_ang_vel_pan equal to the angular velocity
// Assume: my is set to a biped01 entity
function BipedPhy01_Pan(ang_dist)
{
	move_mode = ignore_passable + ignore_passents + ignore_push + activate_trigger + glide + use_box;
	ang_dist = c_rotate(my,vector(ang_dist,0,0),move_mode);

	my._gid01_ang_vel_pan = ang_dist / time_step; // calculate ang vel for use later
}




//==============================================================================================================================================

// desc: simple scan. Does NOT detect passable blocks (water,forcefields, etc.)
//
// Assume: my is set to a valid biped01 entity (see doc for valid skills)
var bipedPhy01_to_vec[3];   // can't use local vecs when calling functions (v6.2)
var bipedPhy01_from_vec[3];
function	BipedPhy01_Scan()  // c_collide version
{
if(my == NULL) { diag("************NULLED MY IN BipedPhy01_Scan****"); return(-1); }

 	// Use trace to scan the surface under the entity
	vec_set(bipedPhy01_from_vec,my.x);
	vec_set(bipedPhy01_to_vec,my.x);
	trace_mode = IGNORE_ME + ACTIVATE_SONAR + USE_BOX;// + IGNORE_PASSABLE + IGNORE_PASSENTS;
	bipedPhy01_to_vec.z -= bipedPhy01_scan_dist_const;
	bipedPhy01_height = c_trace(bipedPhy01_from_vec,bipedPhy01_to_vec,trace_mode);

ifdef	BIPEDPHY01_DEBUG;
if(passable_ent != null) { draw_text("TRACE_ENT",200,200,vector(90,30,233)); }
else { draw_text("TRACE_ZERO",200,200,vector(90,30,233)); }
if(ON_PASSABLE) { draw_text("ON_PASSABLE",200,250,vector(90,30,233)); }
if(IN_PASSABLE) { draw_text("IN_PASSABLE",200,300,vector(90,30,233)); }
endif;

	if(0 == trace_hit) // 0-if nothing hit
	{
		bipedPhy01_height = bipedPhy01_scan_dist_const;	// greater than x units off the ground (in air)
	}


	bipedPhy01_temp_ent = you;	// save you for later use

	vec_set(bipedPhy01_surface_normal_vec.x,NORMAL.X); // normal is set in trace


	if(passable_ent == null)
	{
		if(ON_PASSABLE || IN_PASSABLE)
		{
			// first trace returned a passable block
			trace_mode = IGNORE_ME + ACTIVATE_SONAR + USE_BOX + IGNORE_PASSABLE + IGNORE_PASSENTS;
			bipedPhy01_height = c_trace(bipedPhy01_from_vec,bipedPhy01_to_vec,trace_mode);
		}
	}

//??	bipedPhy01_height -= 2;	// "buffer" to help biped over edges


	// if the player is standing in a forcefield, water, or on a platform
	if(bipedPhy01_temp_ent != NULL)
	{
		if(gid01_platform_const == bipedPhy01_temp_ent._gid01_id)
		{
			// add current platform velocity
			vec_set(bipedPhy01_surface_vel_vec[0],bipedPhy01_temp_ent._gid01_vel_x);
		}
		else
		{
			// no added velocity due to platform
			vec_set(bipedPhy01_surface_vel_vec[0],nullvector);
		}
	}
	else
	{
		// no added velocity
		vec_set(bipedPhy01_surface_vel_vec[0],nullvector);


	}

	return(0);
}



// Desc: move on the ground, air, or wading
//
// assume: my is set to a biped01
//			  bipedPhy01_height is set to the current height above a surface
//			  bipedPhy01_grnd_friction is set to the ground friction value
function	BipedPhy01_Move_Ground(&force_vec, pan_dist)
{

	var	friction;	// friction constant
	var	delta_fric; // friction over time value
	var	dist[3];		// distance covered this frame in direction faced
	var	abs_dist[3];// additional distance covered this frame in world space
	var	abs_force[3];//forces from the environment (gravity, elevators, etc.)
	var	q_jump_or_duck; // 1: jump, -1: duck, 0: neither

	// jump or duck?
	q_jump_or_duck = sign(force_vec[2]);
	force_vec[2] = 0;

	// this biped doesn't tilt or roll
	my.tilt = 0;
	my.roll = 0;

	// rotate the player (absolute)
	BipedPhy01_Pan(pan_dist);

	///////////////////////////////////////////
	// drop to ground and check for collision


	if(bipedPhy01_height<2) // on, in, or near the ground?
	{
		// push out of ground
		// note: With the new collision model, this should never happen (but does, rarely)
		// note2: with new collision improvements, pushing out of the ground can cause more problems then it fixes,
		//    for now it is set to 0.
		vec_set(abs_dist,nullvector);// = 0;//-(bipedPhy01_height);
		my._gid01_vel_z = max(0,my._gid01_vel_z); // stop any down velocity
	}
	else   // gravity is in effect.
	{
		abs_dist.x = 0; abs_dist.y = 0;
 		abs_dist.z = accelerate(my._gid01_vel_z,-bipedPhy01_gravity,0); 						// accel downward
 	}

	// use the move_min_z to limit the slope to which the biped can stand on.
	// NOTE: REMOVED so we can climb steps
//	if(my._move_min_z__003 >= 0) { move_min_z = min(0.99,(1-my._move_min_z__003)); }    // set the min_z glide value


   // Height used to detect on ground or in air
	if(bipedPhy01_height > 3)
	{
		bipedPhy01_height = bipedPhy01_scan_dist_const;	// Still in the air

		friction = bipedPhy01_air_friction; // use air friction

   	// airborne - reduce all relative forces
		// to prevent him from jumping or further moving in the air
		force_vec[0] *= 0.2; // don't set the force completely to zero, otherwise
		force_vec[1] *= 0.2; // player could be stuck on top of a non-wmb entity
		force_vec[2] = 0;

	//--	vec_set(abs_force,nullvector);  // gravity is handled above.


 		// only falling if moving downward (but not if ducking)
		if(my._gid01_vel_z < 0)
		{
			// check to make sure we are not in another state
			if(my._biped01_state_mod != biped01_state_mod_fall_const)
			{
				my._biped01_state_mod = biped01_state_mod_fall_const; // set falling flag
				my._biped01_state_mod_value = 0; // reset falling time
			}
			my._biped01_state_mod_value += time_step;   // add falling time
		}


	}
	else    // on the Ground
	{
		bipedPhy01_height = 0;
		friction = bipedPhy01_grnd_friction; // use ground friction

		//-------------------------------------------------------------------------------------------------------------------------
		// Calculate falling damage
 		if((my._biped01_state_mod == biped01_state_mod_fall_const) &&  // we are falling
			(my._biped01_state_mod_value > bipedPhy01_safe_fall_time_const))	// we've been falling more then the safe amount of time
  		{
			// indicate damage from environment
			my._biped01_state2_mod = biped01_state_mod_damage_const;
			my._biped01_state2_mod_value = BipedPhy01_Fall_Damage();		// take damage depending on fall_time
			my._biped01_state_mod = biped01_state_mod_none_const; // reset flag (we are on ground)
			my._biped01_state_mod_value = 0;
		}
		else
		{
			// note: this works ONLY because no other state can exist at this point
			//(e.g. the player is not underwater and not falling)
			my._biped01_state_mod = biped01_state_mod_none_const; // reset flag (we are on ground)
			my._biped01_state_mod_value = 0;
		}
		//-------------------------------------------------------------------------------------------------------------------------

 		// reset absolute forces
 //--		vec_set(abs_force,nullvector);



		// do we wish to jump?
		if((q_jump_or_duck>0) && (my._biped01_jump_target <= 0)    // no jumping from ducking, or if we are already jumping
		  	&& (bipedPhy01_surface_normal_vec.z >= (my._move_min_z__003)) && (my._biped01_state != biped01_state_wade_const)) // no jumping from slopes or water
		{
			// start a jump
			my._biped01_jump_target = my._jump_height__003; // jump delta
			my._biped01_state = biped01_state_jump_const; // set state to jump
		}
		else
		{
			if(q_jump_or_duck<0)
			{
				if(my._biped01_state != biped01_state_duck_const)
				{
  					my.min_z += my._duck_height__003;

				}
				my._biped01_state = biped01_state_duck_const; // set state to duck
 				vec_scale(force_vec,0.5);
			}
			else
			{
				if(my._biped01_state == biped01_state_duck_const)
				{
					// check to see if we can stand up.
 					// Use trace to scan the surface under the entity
					vec_set(bipedPhy01_from_vec,my.x);
					vec_set(bipedPhy01_to_vec,my.x);
 					trace_mode = IGNORE_ME + IGNORE_PASSENTS + USE_BOX;
					bipedPhy01_to_vec.z += my._duck_height__003 + 1;
					c_trace(bipedPhy01_from_vec,bipedPhy01_to_vec,trace_mode);

					if(0 == trace_hit)  // 0 if nothing hit
					{
						c_move(my,nullvector,vector(0,0,(my._duck_height__003 + 1)),GLIDE);
  						my.min_z -= my._duck_height__003;
						my._biped01_state = biped01_state_walk_const;
					}
					else
					{
						// my._biped01_state = biped01_state_duck_const;  state is already duck
 						vec_scale(force_vec,0.5);
					}

				}
				else
				{
					// walking/standing/running on solid ground
					my._biped01_state = biped01_state_walk_const;
				}
			}
		}



	} // end "on the ground"


	// calculate velocities
	dist.x = accelerate(my._gid01_vel_x,force_vec[0],friction);
	dist.y = accelerate(my._gid01_vel_y,force_vec[1],friction);
	dist.z = 0;                      // dz = 0  (only gravity and jumping)


	if(my._biped01_jump_target <= 0)	// not jumping
	{
	  //	my._gid01_vel_z = (time_step * abs_force.z) + (delta_fric * my._gid01_vel_z);    // gravity force (old way)
		abs_dist.z += accelerate(my._gid01_vel_z,abs_force.z,friction);	// new way
	}
	else	// jumping
	{
	 	my._gid01_vel_z = sqrt(my._biped01_jump_target*2*bipedPhy01_gravity);    // jumping force
		my._biped01_jump_target -= my._gid01_vel_z * time_step;	// reduce the delta
		abs_dist.z += my._gid01_vel_z * time_step;
	}


	// calculate absolute distance to move
	// NOTE:
	//  - bipedPhy01_surface_vel_vec is any velocity added from standing on a moving platform
	abs_dist.x += ((abs_force.x * time_step) + bipedPhy01_surface_vel_vec[0]) * time_step;   // dx = ax*dt^2
	abs_dist.y += ((abs_force.y * time_step) + bipedPhy01_surface_vel_vec[1]) * time_step;   // dy = ay*dt^2
//	abs_dist.z = (my._gid01_vel_z + bipedPhy01_surface_vel_vec[2])* time_step;       // dz = vz*dt
	abs_dist.z += (bipedPhy01_surface_vel_vec[2])* time_step;       // dz = vz*dt


	// 'normal' move
  //	disable_z_glide = 2; // no sliding down
	move_mode = ignore_passable + ignore_passents + ignore_push + activate_trigger + use_box + glide;
	result = c_move(my,dist,abs_dist,move_mode); // ground (and air) move..
ifdef	BIPEDPHY01_DEBUG;
vec_set(bipedPhy01_debug_data[12],dist);
vec_set(bipedPhy01_debug_data[15],abs_dist);
bipedPhy01_debug_data[18] = my._move_min_z__003;
endif;

  //	move_min_z = 0; // reset value
  //	disable_z_glide = 0;


	// calculate distance for animation
	if(result > 0)
	{
		// only use the relative distance traveled (for animation)
		my._biped01_dist = vec_length(dist);
		my._biped01_dist = min(my._biped01_dist,result);
		if(force_vec[0] < 0) { my._biped01_dist *= -1; }	// walking backwards
	}
	else
	{
		// player is not moving, do not animate
		my._biped01_dist = 0;
	}

}


// Desc: move in water (passable) surfaces
function	BipedPhy01_Move_Water(&force_vec, pan_dist)
{
	var	dist[3];		// distance covered this frame in direction faced
	var	abs_dist[3];// additional distance covered this frame in world space
	var	q_up_or_down; // 1: up, -1: down, 0: neither

	// up or down?
	q_up_or_down = sign(force_vec[2]);


	my._biped01_state = biped01_state_swim_const;

	// rotate the player (absolute)
	BipedPhy01_Pan(pan_dist);

	vec_set(abs_dist.x,nullvector);

	// calculate velocities
	vec_accelerate(dist.x,my._gid01_vel_x,force_vec,bipedPhy01_water_friction);

 	// calculate absolute distance to move
	// NOTE:
	//  - we are only concerned about the entity forces here
	if(bipedPhy01_content_ent != NULL)
	{
		vec_add(abs_dist.x,bipedPhy01_content_ent._gid01_vel_x);
		vec_scale(abs_dist.x,time_step);
	}


	// check for surface swimming
	if(ON_PASSABLE == ON)
	{
		// add in gravity
		abs_dist.z += accelerate(my._gid01_vel_z,-bipedPhy01_gravity,0);

		// no upward forces
	   abs_dist.z = min(0,abs_dist.z);
		dist.z = min(0,dist.z);
	}
	else
	{
		// Reduce any speed from falling
		abs_dist.z += accelerate(my._gid01_vel_z,0,0.85); // reduce speed
	}

	move_mode = ignore_passable + ignore_passents + ignore_push + activate_trigger + use_box + glide;
	result = c_move(my,dist,abs_dist,move_mode); // ground (and air) move..

// 	move_min_z = 0; // reset value

	// calculate distance for animation
	if(result > 0)
	{
		// only use the relative distance traveled (for animation)
		my._biped01_dist = vec_length(dist);
		my._biped01_dist = min(my._biped01_dist,result);
		if(force_vec[0] < 0) { my._biped01_dist *= -1; }	// swimming backwards
	}
	else
	{
		// player is not moving, do not animate
		my._biped01_dist = 0;
	}

}


// =====================================================================
// UPDATE FUNCTION

// Desc: update a biped entity for a frame
//
// Assume: 'my' is set to a valid biped01 entity (see doc for valid skills)
function	BipedPhy01_Update(&force_vec, pan_dist)
{
	var	biped_content;	// 1-empty, 2-passable (water), 3-solid

	if(my._biped01_state == biped01_state_override_const)
	{
		// movement is being overriden at this time
		return(biped01_state_override_const);
	}

	// if we haven't set a state yet, set the state to walking (default state)
	if(my._biped01_state == biped01_state_none_const)
	{
		my._biped01_state = biped01_state_walk_const;
	}


	// scan floor and check to see if we are in solid, passable, etc.
	// sets: bipedPhy01_height: height above nearest surface
	//		   bipedPhy01_surface_normal_vec[3]: normal vector of surface
	biped_content = BipedPhy01_Scan();

ifdef	BIPEDPHY01_DEBUG;
bipedPhy01_debug_data[0] = bipedPhy01_height;
bipedPhy01_debug_data[1] = trace_hit;
bipedPhy01_debug_data[2] = my._biped01_state;
bipedPhy01_debug_data[3] = 100 * IN_SOLID + 10 * ON_PASSABLE + IN_PASSABLE;
vec_set( bipedPhy01_debug_data[5],bipedPhy01_surface_normal_vec);
vec_set( bipedPhy01_debug_data[8],force_vec);
bipedPhy01_debug_data[11] = pan_dist;
BipedPhy01_Vis_Col();
endif;

	// select rotation and move function by content
 //---	c_content(my.x,0);
	if((IN_PASSABLE == ON) || (ON_PASSABLE))
	{
		bipedPhy01_content_ent = passable_ent;//you;
	}
	else
	{
		bipedPhy01_content_ent = NULL;
	}

	if(bipedPhy01_content_ent != null)
	{
		if(gid01_water_const == bipedPhy01_content_ent._gid01_id)
		{
			BipedPhy01_Move_Water(force_vec,pan_dist);

		}
		else
		{
   		BipedPhy01_Move_Ground(force_vec,pan_dist);
		}
	}
	else
	{
   	BipedPhy01_Move_Ground(force_vec,pan_dist);
	}

	//+++ clean up

}



ifdef	BIPEDPHY01_DEBUG;
include <debug01.wdl>;


// Desc: Draw a collision sphere used in c_move
//
// Note: This function only handles non-rotated bounding spheres.
//			It is only used as a rough estimate of the actual bounding sphere.
//
//	Note2: This is an "ugly hack" and is ment for debugging only!
var	debug01_old_pt[3];
var	debug01_new_pt[3];
var	debug01_old_pt2[3]; 	// mirror points
var	debug01_new_pt2[3];  // mirror points
function	BipedPhy01_Vis_Col()
{
	var	pt[3];
	var	c_pt[3];
	var	const[3];	//
	var	off_c_pt[3];
	var	old_x;
	var	old_y;

	// calculate center point
	c_pt.x = (my.max_x + my.min_x)/2;
	c_pt.y = (my.max_y + my.min_y)/2;
	c_pt.z = (my.max_z + my.min_z)/2;

	// calculate the offset centerpoint using rotation
	vec_set(off_c_pt,c_pt);
	vec_rotate(off_c_pt,my.pan);

	const.x = my.max_x - c_pt.x;

	Debug01_Draw_Line(vector(my.x+off_c_pt.x+const.x,my.y+off_c_pt.y,my.z+off_c_pt.z),
							vector(my.x+off_c_pt.x-const.x,my.y+off_c_pt.y,my.z+off_c_pt.z));

	const.x *= const.x;
	const.y = my.max_y - c_pt.y;

	Debug01_Draw_Line(vector(my.x+off_c_pt.x,my.y+off_c_pt.y+const.y,my.z+off_c_pt.z),
							vector(my.x+off_c_pt.x,my.y+off_c_pt.y-const.y,my.z+off_c_pt.z));

	const.y *= const.y;
	const.z = my.max_z - c_pt.z;
	const.z *= const.z;



	// ==========================================
	// Front/Back
	vec_set(debug01_new_pt,off_c_pt);
	vec_add(debug01_new_pt,my.x);
	vec_set(debug01_new_pt2,debug01_new_pt);
	pt.y = 0;
	pt.z = my.max_z - c_pt.z;
	while(pt.z > -(my.max_z - c_pt.z))
	{
		old_x = pt.x;
		pt.x = sqrt( (const.x*(1- ( (pt.z*pt.z)/(const.z) ) )) );

		vec_set(debug01_old_pt,debug01_new_pt);// save old
		vec_set(debug01_new_pt,pt);            // create new
		vec_rotate(debug01_new_pt,my.pan);		// rotate by player angle
		// use calculations to find back arc
		vec_set(debug01_old_pt2,debug01_new_pt2); 		// save old
		vec_set(debug01_new_pt2,debug01_new_pt);  		// create new
		vec_rotate(debug01_new_pt2,vector(180,0,0));		// rotate by 180
		vec_add(debug01_new_pt,my.x);          // offset by player position
		vec_add(debug01_new_pt2,my.x);         // offset by player position
		vec_add(debug01_new_pt,off_c_pt);      // offset by centerpoint
		vec_add(debug01_new_pt2,off_c_pt);     // offset by centerpoint
		Debug01_Draw_Line(debug01_old_pt,debug01_new_pt);  // front arc
		Debug01_Draw_Line(debug01_old_pt2,debug01_new_pt2);  // back arc

		pt.z -= 5;
	}
 	old_x = pt.x;
	pt.z = -(my.max_z - c_pt.z);
 	pt.x = sqrt( (const.x*(1- ( (pt.z*pt.z)/(const.z) ) )) );

	vec_set(debug01_old_pt,debug01_new_pt);// save old
	vec_set(debug01_new_pt,pt);            // create new
	vec_rotate(debug01_new_pt,my.pan);		// rotate by player angle
	// use calculations to find back arc
	vec_set(debug01_old_pt2,debug01_new_pt2); 		// save old
	vec_set(debug01_new_pt2,debug01_new_pt);  		// create new
	vec_rotate(debug01_new_pt2,vector(180,0,0));		// rotate by 180
	vec_add(debug01_new_pt,my.x);          // offset by player position
	vec_add(debug01_new_pt2,my.x);         // offset by player position
	vec_add(debug01_new_pt,off_c_pt);      // offset by centerpoint
	vec_add(debug01_new_pt2,off_c_pt);     // offset by centerpoint
	Debug01_Draw_Line(debug01_old_pt,debug01_new_pt);  // front arc
	Debug01_Draw_Line(debug01_old_pt2,debug01_new_pt2);  // back arc

	// =====================================
	// Left/Right
	pt.x = 0;
	pt.z = my.max_z - c_pt.z;
	while(pt.z > -(my.max_z - c_pt.z))
	{
		old_y = pt.y;
		pt.y = sqrt( (const.y*(1- ( (pt.z*pt.z)/(const.z) ) )) );

		vec_set(debug01_old_pt,debug01_new_pt);// save old
		vec_set(debug01_new_pt,pt);            // create new
		vec_rotate(debug01_new_pt,my.pan);		// rotate by player angle
		// use calculations to find back arc
		vec_set(debug01_old_pt2,debug01_new_pt2); 		// save old
		vec_set(debug01_new_pt2,debug01_new_pt);  		// create new
		vec_rotate(debug01_new_pt2,vector(180,0,0));		// rotate by 180
		vec_add(debug01_new_pt,my.x);          // offset by player position
		vec_add(debug01_new_pt2,my.x);         // offset by player position
		vec_add(debug01_new_pt,off_c_pt);      // offset by centerpoint
		vec_add(debug01_new_pt2,off_c_pt);     // offset by centerpoint
		Debug01_Draw_Line(debug01_old_pt,debug01_new_pt);  // front arc
		Debug01_Draw_Line(debug01_old_pt2,debug01_new_pt2);  // back arc
		pt.z -= 5;

	}
 	old_y = pt.y;
	pt.z = -(my.max_z - c_pt.z);
 	pt.y = sqrt( (const.y*(1- ( (pt.z*pt.z)/(const.z) ) )) );

	vec_set(debug01_old_pt,debug01_new_pt);// save old
	vec_set(debug01_new_pt,pt);            // create new
	vec_rotate(debug01_new_pt,my.pan);		// rotate by player angle
	// use calculations to find back arc
	vec_set(debug01_old_pt2,debug01_new_pt2); 		// save old
	vec_set(debug01_new_pt2,debug01_new_pt);  		// create new
	vec_rotate(debug01_new_pt2,vector(180,0,0));		// rotate by 180
	vec_add(debug01_new_pt,my.x);          // offset by player position
	vec_add(debug01_new_pt2,my.x);         // offset by player position
	vec_add(debug01_new_pt,off_c_pt);      // offset by centerpoint
	vec_add(debug01_new_pt2,off_c_pt);     // offset by centerpoint
	Debug01_Draw_Line(debug01_old_pt,debug01_new_pt);  // front arc
	Debug01_Draw_Line(debug01_old_pt2,debug01_new_pt2);  // back arc

	// scan trace line
	Debug01_Draw_Line(my.x,vector(my.x,my.y,my.z+(2*my.min_z-6)));


	my.transparent = on;


}
endif;   // BIPEDPHY01_DEBUG


endif;// bipedPhy01_wdl