ifndef lift_wdl;
define lift_wdl;

// ---------------------------------------------------------------------
// STARTHEADER
//
// ver: 5.0
// engineReq: 6.4
// date: 061011
//
// title: Lifts (01)
// class: OBJECT
// type: USER
// image: lift.pcx
// help: Allows you to add simple moving objects that can be used to carry
/// certain entities around. These objects can be used for platforms, elevators,
/// simple vehicles (e.g. ferryboats, trams, etc.), etc.
//
// needs: gid01.wdl, plSelect.wdl, trigger00.wdl
//
// external:
//
// prefix: lift01_
// idcode: 00E
//
// ENDHEADER
// ---------------------------------------------------------------------


// section: Sounds
// entry: Inactive (locked) Lift Sound
// help: Sound played when a locked door is activated without a key.
// id: 3
sound lift01_locked_snd = <locked.wav>;

// entry: Lift Inactive (locked) Sound Volume
// id: 4
define lift01_locked_snd_vol = 150;

// entry: Lift Activated Sound
// help: Sound played when a lift is activated.
// id: 1
sound lift01_activate_snd = <lift.wav>;

// entry: Lift Moving Sound Volume
// id: 2
define lift01_activate_snd_vol = 200;


// lift01 skills
define	_endpos_x__00E,skill1;   	// position #2
define	_endpos_y__00E,skill2;
define	_endpos_z__00E,skill3;
define	_speed__00E,skill4;         // speed in quants/tick
define	_rest_time__00E,skill5;		// time spent "resting" at each point (seconds)

define	_push__00E, skill6; // "PUSH" value (note! overwriten by _startpos_x__00E after use!)
define	_detect_player__00E, skill7;	// Detect/damage player? (note! moved after to skill30 after init)
define	_detect_player_damage__00E, skill30;	// see _detect_player__00E above
define	_link_id__00E, SKIll8;		// used to link the actions of this lift to one or more Lift01_Linked objects
define	_link_id_actual__00E, SKILL31;// actual place where link id is stored (other is overwriten)

define	_startpos_x__00E,skill6;    // postion #1
define	_startpos_y__00E,skill7;
define	_startpos_z__00E,skill8;

define	_dist_x__00E,skill9;			// distance between points
define	_dist_y__00E,skill10;
define	_dist_z__00E,skill11;

define	_use_trigger__00E,skill12;		// id number of the trigger used to open/close door

define	_use_key__00E,skill13;			// requires a key to be picked up before activating

define	_time_to_target__00E, skill15;


// use global skills so other scripts can read this value
/*
define	_gid01_vel_x, skill21;        // current velocity (quants/tick)
define	_gid01_vel_y, skill22;        // current velocity (quants/tick)
define	_gid01_vel_z, skill23;        // current velocity (quants/tick)
*/

define	_lift01_state,skill20;     			// current state:
define 	lift01_state_at_start_const,1;    	// resting at start point
define 	lift01_state_at_end_const,2;    		// resting at end point
define 	lift01_state_towards_end_const,3;  	// moving towards the end point
define 	lift01_state_towards_start_const,4;	// moving towards the start point
define 	lift01_state_waiting_start_const,5;	// temporarily inactive
define 	lift01_state_waiting_end_const,6;	// temporarily inactive
define	lift01_state_abort_const,7;			// abort movement
define	lift01_state_stop_const,8;				// stopped

define	_use_absolute__00E,flag1;	// use absolute coordinates
define	_oscillate__00E,   flag2;  // oscillate between the two points
define	_ignore_scan__00E, flag3;  // ignore scans
define	_ignore_sonar__00E,flag4;	// ignore sonar
define	_ignore_click__00E,flag5;  // ignore left mouse click
define   _ignore_map__00E,  flag6;	// ignores all map entities.
define	_auto_return__00E, flag7;  // return to start after wait time
define   _no_sound__00E,    flag8;	// don't make sound





// Desc: move lift to end position
function	Lift01_Move_End()
{
	// Note: since other objects may be moved by this object, it should go last.
//	proc_late();

	// move towards end position
	my._lift01_state = lift01_state_towards_end_const;

	if(my._no_sound__00E == OFF)
	{
		// play lift sound
		ent_playsound(my,lift01_activate_snd,lift01_activate_snd_vol);
	}

	// calculate delta to target
//	vec_diff(my._dist_x__00E,my._endpos_x__00E,my._startpos_x__00E);  // dist = start->end
	vec_diff(my._dist_x__00E,my._endpos_x__00E,my.x);  // dist = current->end
	vec_set(my._gid01_vel_x,my._dist_x__00E);
	if(my._speed__00E <= 0) { my._speed__00E = 1; }
	vec_normalize(my._gid01_vel_x,my._speed__00E);	// delta per tick

	// calculate "ticks to target"
	my._time_to_target__00E = vec_length(my._dist_x__00E)/my._speed__00E;

	wait(1);	// wait a frame before movment

	while(my._time_to_target__00E > time_step)
	{
		// temp is this frame's movement vector
		vec_set(temp.x,my._gid01_vel_x);
		vec_scale(temp.x,time_step);

		// check for abort!
		if(my._gid01_message > 0)
		{
			if(my._gid01_message > 1)
			{
				my._lift01_state = lift01_state_waiting_end_const; // signal finish
				my._gid01_message = 0;                     	// reset flag
				return;
			}
			my._lift01_state = lift01_state_abort_const; // signal abort
			my._gid01_message = 0;                     	// reset flag
			return;
		}

		// send message to player if we hit them
		if(my._detect_player_damage__00E != 0)
		{
 			// alert player if hit
			gid01_event_type = gid01_event_push;
 			// apply damage
			gid01_event_value -= abs(my._detect_player_damage__00E) - 1;
		}
		move_mode = ignore_passable + ignore_passents + ignore_push;
		if(my._ignore_map__00E == ON) { move_mode += ignore_maps; }
		c_move(my,nullvector,temp.x,move_mode);
		// Clear event flag!!!
		gid01_event_type = gid01_event_null;

		my._time_to_target__00E -= time_step;
		wait(1);	// move every frame
	}

	// snap to end point
	vec_diff(temp.x,my._endpos_x__00E,my.x);  // temp = my->end

	// check for abort!
	if(my._gid01_message > 0)
	{
		if(my._gid01_message > 1)
		{
			my._lift01_state = lift01_state_waiting_end_const; // signal finish
			my._gid01_message = 0;                     	// reset flag
			return;
		}
		my._lift01_state = lift01_state_abort_const; // signal abort
		my._gid01_message = 0;                     	// reset flag
		return;
	}

	// send message to player if we hit them
	if(my._detect_player_damage__00E != 0)
	{
 		// alert player if hit
		gid01_event_type = gid01_event_push;
 		// apply damage
		gid01_event_value -= abs(my._detect_player_damage__00E) - 1;
	}
	move_mode = ignore_passable + ignore_passents + ignore_push;
	if(my._ignore_map__00E == ON) { move_mode += ignore_maps; }
	c_move(my,nullvector,temp.x,move_mode);
	// Clear event flag!!!
	gid01_event_type = gid01_event_null;

	vec_scale(my._gid01_vel_x,1/time_step);  // set velocity (used by other functions)


	// done moving
	vec_set(my._gid01_vel_x,nullvector);  // done moving
	my._lift01_state = lift01_state_waiting_end_const;  // in 'wait' state
}




function	Lift01_Move_Start()
{
	// Note: since other objects may be moved by this object, it should go last.
//	proc_late();

	// move towards start position
   my._lift01_state = lift01_state_towards_start_const;

	if(my._no_sound__00E == OFF)
	{
		// play lift sound
		ent_playsound(my,lift01_activate_snd,lift01_activate_snd_vol);
	}

	// calculate delta to target
//	vec_diff(my._dist_x__00E,my._startpos_x__00E, my._endpos_x__00E);  // dist = end->start
	vec_diff(my._dist_x__00E,my._startpos_x__00E, my.x);  // dist = current->start
	vec_set(my._gid01_vel_x,my._dist_x__00E);

	if(my._speed__00E <= 0) { my._speed__00E = 1; }
	vec_normalize(my._gid01_vel_x,my._speed__00E);	// delta per tick

	// calculate "ticks to target"
	my._time_to_target__00E = vec_length(my._dist_x__00E)/my._speed__00E;

	wait(1);	// wait a frame before movment

	while(my._time_to_target__00E > time_step)
	{
		// temp is this frame's movement vector
		vec_set(temp.x,my._gid01_vel_x);
		vec_scale(temp.x,time_step);

		// check for abort!
		if(my._gid01_message > 0)
		{
			if(my._gid01_message > 1)
			{
				my._lift01_state = lift01_state_waiting_start_const; // signal finish
				my._gid01_message = 0;                     	// reset flag
				return;
			}
			my._gid01_message = 0;                     	// reset flag
			my._lift01_state = lift01_state_abort_const; // signal abort
			return;
		}

		// send message to player if we hit them
		if(my._detect_player_damage__00E != 0)
		{
 			// alert player if hit
			gid01_event_type = gid01_event_push;
 			// apply damage
			gid01_event_value -= abs(my._detect_player_damage__00E) - 1;
		}
		move_mode = ignore_passable + ignore_passents + ignore_push;
		if(my._ignore_map__00E == ON) { move_mode += ignore_maps; }
		c_move(my,nullvector,temp.x,move_mode);
		// Clear event flag!!!
		gid01_event_type = gid01_event_null;

		my._time_to_target__00E -= time_step;
		wait(1);	// move every frame
	}

	// snap to start point
	vec_diff(temp.x,my._startpos_x__00E,my.x);  // temp = my->start
	// check for abort!
	if(my._gid01_message > 0)
	{
		if(my._gid01_message > 1)
		{
			my._lift01_state = lift01_state_waiting_start_const; // signal finish
			my._gid01_message = 0;                     	// reset flag
			return;
		}

		my._lift01_state = lift01_state_abort_const; // signal abort
		my._gid01_message = 0;                     	// reset flag
		return;
	}

	// send message to player if we hit them
	if(my._detect_player_damage__00E != 0)
	{
 		// alert player if hit
		gid01_event_type = gid01_event_push;
 		// apply damage
		gid01_event_value -= abs(my._detect_player_damage__00E) - 1;
	}
	move_mode = ignore_passable + ignore_passents + ignore_push;
	if(my._ignore_map__00E == ON) { move_mode += ignore_maps; }
	c_move(my,nullvector,temp.x,move_mode);
	// Clear event flag!!!
	gid01_event_type = gid01_event_null;

 	// done moving
	vec_set(my._gid01_vel_x,nullvector);  // done moving
	my._lift01_state = lift01_state_waiting_start_const; // wait state
}


// Desc: Handle lift activation
function	Lift01_Activate()
{
	// resting at start point
	if(my._lift01_state == lift01_state_at_start_const)
	{
		Lift01_Move_End();
		while(my._lift01_state == lift01_state_towards_end_const) { wait(1); }
		if(my._lift01_state == lift01_state_abort_const)
		{
			// hit something (like the player)
			my._lift01_state = lift01_state_at_end_const;	// move back
			Lift01_Activate();
			return;

		}

		if(my._use_trigger__00E < 0) 	// reset trigger at end?
		{
			Trigger00_Reset((-my._use_trigger__00E),trigger00_bitlist);
		}

		// wait for pre-set amount of time
		sleep(my._rest_time__00E);

		my._lift01_state = lift01_state_at_end_const;  // flag at end


		if(my._auto_return__00E != ON)
		{
			// no auto return function
			return;
		}

	}

	// resting at end point
	if(my._lift01_state == lift01_state_at_end_const)
	{
		Lift01_Move_Start();
		while(my._lift01_state == lift01_state_towards_start_const) { wait(1); }
		if(my._lift01_state == lift01_state_abort_const)
		{
			// hit something (like the player)
			my._lift01_state = lift01_state_at_start_const;	// move back
			Lift01_Activate();
			return;
		}

		// wait for pre-set amount of time
		sleep(my._rest_time__00E);

  		my._lift01_state = lift01_state_at_start_const;
		return;
	}

	// not at start or end point...
	// do nothing.

}

// Desc: handle events sent to a basic lift
function	 Lift01_Basic_Event()
{
	if(event_scan == event_type)
	{
		// only activate on "activate" scans
		if(gid01_event_scan_activate != gid01_event_type)
 		{
			return;
		}
	}



	if(my._use_key__00E > 0)
	{
		// check for key
		if(Trigger00_Check(my._use_key__00E,trigger00_bitlist) != 1)
		{
			if(my._no_sound__00E == OFF)
			{
				// play lift locked sound
				ent_playsound(my,lift01_locked_snd,lift01_locked_snd_vol);
			}

			return;
		}
	}

	Lift01_Activate();	// activate the lift
}

// Desc:  move to end if trigger == ON
//			 move to start if trigger == OFF
function Lift01_Check_Trigger()
{
	var	trigger_id;

	trigger_id = abs(my._use_trigger__00E);

	while(1)
	{
		wait(1);
		if(Trigger00_Check(trigger_id,trigger00_bitlist) == 1)
		{
			if(my._lift01_state == lift01_state_at_start_const)
			{
					Lift01_Activate(); // move to end..
			}
		}
		else
		{
			if(Trigger00_Check(trigger_id,trigger00_bitlist) == 0)
			{
				if(my._lift01_state == lift01_state_at_end_const)
				{
					Lift01_Activate(); // return..
				}
			}
		}

	} // end loop
}


// Desc: find linked ent
function Lift01_Find_Link(id)
{
	you = ent_next(NULL); // retrieve first entity
 	while(you != NULL) // repeat until there are no more entities
	{
		if(you._gid01_id == gid01_platform_const)
		{
			if(you._link_id_actual__00E == id)
			{
				return(handle(you));	// you equal the first linked lift found
				// note: we must return a handle
			}
		}
		you = ent_next(you); // get next entity
 	}
	return(0);
}


// action: Lift01_Linked
// title: Linked Lift
// Desc: Activates when the lift it is attached to activates.
// Help: You can use either link ID or attach (in Properties)
//
// uses _endpos_x__00E, _endpos_y__00E, _endpos_z__00E,
// uses _use_absolute__00E, _speed__00E
// uses _link_id__00E
// uses _ignore_map__00E
// uses _no_sound__00E
//
//------------------
// section: Target Pos
//
//	skill1: EndX 0
//	skill2: EndY 0
//	skill3: EndZ 0
// skill4: Speed 5
//
// skill8: LinkID 0
// help: Link to other lift (if attach not used).
//
// flag1: Absolute? 0
// help: Use absolute pos?
//
// flag6: IgnoreMap? 1
//
// flag8: NoSound 0
//
action	Lift01_Linked
{
	while(gid01_level_state != gid01_level_loaded) { wait(1); }
	wait(3);	// make sure everything is loaded

	if(my.parent == NULL)
	{
		if(my._link_id__00e <= 0)
		{
			diag("\nWARNING! Lift01::Lift01_Linked - Bad link id. Entity removed");
			ent_remove(me);
			return;
		}

		// Link to buddy
		my.skill33 = Lift01_Find_Link(my._link_id__00E);	// set you to link buddy
		if(my.skill33 == 0)
		{
			diag("\nWARNING! Lift01::Lift01_Linked - No parent lift found for link. Entity removed");
			ent_remove(me);
			return;
		}
		my.parent  = ptr_for_handle(my.skill33);
	}
	else
	{
		my.skill33 = handle(my.parent);
	}

	you = my.parent;	// we do this because c-script doesn't allow for my.parent.push, etc.


	// initialize the movement parameters of the elevator
	my.push = you.push;
	my._detect_player_damage__00E = you._detect_player_damage__00E;

	if(my._speed__00E == 0) { my._speed__00E = you._speed__00E; }


	vec_set(my._startpos_x__00E,my.x);     // start point

	if(off == my._use_absolute__00E)	// use absolute coordinate flag not set?
	{
		vec_add(my._endpos_x__00E,my.x);
	}
	// otherwise, use the coordinate stored inside the skills at the target point

 	// by definition, we are at the starting point
	my._lift01_state = lift01_state_at_start_const;



	while(you != NULL)
	{

		// ignore equal states (which should be true most of the time)
		if(my._lift01_state != you._lift01_state)
		{

			// waiting at start
			if(my._lift01_state == lift01_state_at_start_const)
			{
				if((you._lift01_state == lift01_state_at_end_const)
					|| (you._lift01_state == lift01_state_towards_end_const)
					|| (you._lift01_state == lift01_state_waiting_end_const))
				{
					Lift01_Move_End();
				}
			}

			// waiting at end
			if(my._lift01_state == lift01_state_at_end_const)
			{
				if((you._lift01_state == lift01_state_at_start_const)
					|| (you._lift01_state == lift01_state_towards_start_const)
					|| (you._lift01_state == lift01_state_waiting_start_const))
				{
					Lift01_Move_Start();
				}
			}


			// We've been stopped for some reason.
			// note! Must come BEFORE this state is set (otherwise lift will just continue)
			if(my._lift01_state == lift01_state_stop_const)
			{
				if((you._lift01_state == lift01_state_at_end_const)
					|| (you._lift01_state == lift01_state_towards_end_const)
					|| (you._lift01_state == lift01_state_waiting_end_const))
				{
					Lift01_Move_End();
				}
				else
				{
				 	if((you._lift01_state == lift01_state_at_start_const)
						|| (you._lift01_state == lift01_state_towards_start_const)
						|| (you._lift01_state == lift01_state_waiting_start_const))
					{
						Lift01_Move_Start();
					}
				}
			}

			if(my._lift01_state == lift01_state_abort_const )
			{
				// send message to link buddy
				you._gid01_message = 1;
				// note: no need to react here, link buddy will lead the way
				my._lift01_state = lift01_state_stop_const;
			}



			if(my._lift01_state == lift01_state_towards_start_const)
			{
				if((you._lift01_state == lift01_state_at_end_const)
					|| (you._lift01_state == lift01_state_towards_end_const)
					|| (you._lift01_state == lift01_state_waiting_end_const))
				{
					// send message to self to finish movement
					my._gid01_message = 2;
				}
			}


			if(my._lift01_state == lift01_state_towards_end_const)
			{
				if((you._lift01_state == lift01_state_at_start_const)
					|| (you._lift01_state == lift01_state_towards_start_const)
					|| (you._lift01_state == lift01_state_waiting_start_const))
				{
					// send message to self to finish movement
					my._gid01_message = 2;
				}
			}

			// we don't have a 'wait' state (always follow link buddy)
			if(my._lift01_state == lift01_state_waiting_end_const)
			{
				my._lift01_state = lift01_state_at_end_const;
			}
			if(my._lift01_state == lift01_state_waiting_start_const)
			{
				my._lift01_state = lift01_state_at_start_const;
			}
		} // END if(my._lift01_state != you._lift01_state)

		wait(1);
		you = ptr_for_handle(my.skill33); // restore you pointer
	}
	ent_remove(me);
}


// action: Lift01_Basic
// title: Basic Lift
// Desc: Basic lift action. The entity moves between two points at a fixed speed.
//
// uses _endpos_x__00E, _endpos_y__00E, _endpos_z__00E,
// uses _speed__00E, _use_absolute__00E, _oscillate__00E, _auto_return__00E
// uses _push__00E
// uses _ignore_scan__00E, _ignore_sonar__00E, _ignore_click__00E, _ignore_map__00E
// uses _no_sound__00E
// uses _rest_time__00E, _use_trigger__00E
// uses _detect_player__00E
// uses _link_id__00E, _use_key__00E
//
//------------------
// section: Target Pos
//
//	skill1: EndX 0
//	skill2: EndY 0
//	skill3: EndZ 0
// skill4: Speed 5
// skill5: RestTime 2
//
// flag1: Absolute? 0
// help: Use absolute pos?
//
// flag2: Oscillate? 0
//
// flag7: AutoReturn? 0
//
// flag6: IgnoreMap? 1
//
// -----------------
// section: Activate
// skill12: Trigger 0
// help: If 0, this lift doesn't use a trigger
// help: Negitive values reset the trigger after use.
// cntl: spin x x 1
//
// skill13: Key 0
// help: Trigger of same value must be set before this lift can be activated.
// help: A zero (0) value means it can be activated at any time.
//
// flag3: IgnoreScan 0
//
// flag4: IgnoreSonar 0
//
// flag5: IgnoreClick 0
//
//
// ------------------
// section: Misc
// skill6: Push 10
// help: "PUSH" value (note! overwriten by _startrot_pan__00B after use!)
//
// flag8: NoSound 0
//
// skill8: LinkedLift 0
// help: Any "Linked Lift" with this number will activate at the same time.
//
action Lift01_Basic
{
	// identify ourselves globally as a moving platform
	my._gid01_id = gid01_platform_const;

	my._link_id_actual__00E = my._link_id__00E;	// user value overwriten

	if(my._speed__00E == 0) { my._speed__00E = 5; }


 	my.event = Lift01_Basic_Event;

	// move skill
	my._detect_player_damage__00E = my._detect_player__00E;

	if(my._push__00E == 0)
	{
		my.push = 10;
	}
	else
	{

		my.push = my._push__00E;
	}
	// NOTE!  my._push__00E is no longer valid past this point!!!

	// initialize the movement parameters of the elevator

	vec_set(my._startpos_x__00E,my.x);     // start point

	if(off == my._use_absolute__00E)	// use absolute coordinate flag not set?
	{
		vec_add(my._endpos_x__00E,my.x);
	}
	// otherwise, use the coordinate stored inside the skills at the target point

	// set up events that the lift will react to
	if(off == my._ignore_scan__00E)  {my.enable_scan = on;}	// activate on scan
	if(off == my._ignore_click__00E) {my.enable_click = on;}	// activate on left mouse click
	if(off == my._ignore_sonar__00E) {my.enable_sonar = on;}	// activate on sonar

	// by definition, we are at the starting point
	my._lift01_state = lift01_state_at_start_const;


	// do we use a trigger?
	if(my._use_trigger__00E != 0)  { Lift01_Check_Trigger(); }

	// if we have an oscillating lift...
	while(my._oscillate__00E)
	{
		Lift01_Activate();
		wait(1);
		while((my._lift01_state != lift01_state_at_start_const)
				&& (my._lift01_state != lift01_state_at_end_const))
		{
			// lift is moving
			wait(1);
		}
	}

}

define _light_range__00E, skill13;
define _light_red__00E, skill14;
define _light_green__00E, skill15;
define _light_blue__00E, skill16;
define _light_offset_x__00E, skill17;
define _light_offset_y__00E, skill18;
define _light_offset_z__00E, skill19;

function	Lift01_Light()
{
	my.lightrange = you._light_range__00E;
	my.red = you._light_red__00E;
	my.green = you._light_green__00E;
	my.blue = you._light_blue__00E;

	my.invisible = on;
	my.passable = on;

	vec_set(my.skill1,you._light_offset_x__00E);

	while(you != NULL)
	{
		// move with lift
		vec_set(my.x,you.x);
		vec_add(my.x,my.skill1);
		wait(1);
	}
	wait(1);
	ent_remove(me);

}

// action: Lift01_Lite
// title: Lift with Light
// Desc: Basic lift action plus Light.
//
// uses _light_range__00E, _light_red__00E, _light_green__00E, _light_blue__00E,
// uses _light_offset_x__00E, _light_offset_y__00E, _light_offset_z__00E,
// uses _endpos_x__00E, _endpos_y__00E, _endpos_z__00E,
// uses _speed__00E, _use_absolute__00E, _oscillate__00E, _auto_return__00E
// uses _push__00E
// uses _no_sound__00E
// uses _ignore_scan__00E, _ignore_sonar__00E, _ignore_click__00E, _ignore_map__00E
// uses _rest_time__00E, _use_trigger__00E
// uses _detect_player__00E
//------------------
// section: Target Pos
//
//	skill1: EndX 0
//	skill2: EndY 0
//	skill3: EndZ 0
// skill4: Speed 5
// skill5: RestTime 2
//
// flag1: Absolute? 0
// help: Use absolute pos?
//
// flag2: Oscillate? 0
//
// flag7: AutoReturn? 0
//
// flag6: IgnoreMap? 1
//
// -----------------
// section: Activate
// skill12: Trigger 0
// help: If 0, this door doesn't use a trigger
//
// flag3: IgnoreScan 0
//
// flag4: IgnoreSonar 0
//
// flag5: IgnoreClick 0
//
// ------------------
// section: Light
// skill13: Range 250
// skill14: Red 200
// skill15: Green 200
// skill16: Blue 200
// skill17: OffsetX 0
// skill18: OffsetY 0
// skill19: OffsetZ 0
//
// ------------------
// section: Misc
// skill6: Push 10
// help: "PUSH" value (note! overwriten by _startrot_pan__00B after use!)
//
// flag8: NoSound 0
//
action Lift01_Lite
{
	// Add light
	vec_set(temp.x,my.x);
	vec_add(temp.x,my._light_offset_x__00E);
	ent_create(gid01_temp_mdl,temp,Lift01_Light);

	// we have to zero this value out, because it is also used for keys
	my._light_range__00E = 0;

	// set up basic lift
	Lift01_Basic();
}





endif;