ifndef particle_wdl;
define particle_wdl;

// ---------------------------------------------------------------------
// STARTHEADER
//
// ver: 5.0
// engineReq: 6.4
// date: 061011
//
// title: Particle (00)
// class: ENVIR
// type: USER
// image: particle00.pcx
// help: Handle particle and lighting effects.
//
// needs: gid01.wdl, plSelect.wdl, trigger00.wdl
//
// prefix: particle00_
// idcode: 011
//
// ENDHEADER
// ---------------------------------------------------------------------


// ---------------------------------------------------------------------
// SKILLS
define	_rate__011,SKILL1;	// ticks between paricles
define	_use_trigger__011,SKILL2; // activating trigger (0-always on)
define	_vel_x__011,SKILL4;	// base velocity
define	_vel_y__011,SKILL5;	// base velocity
define	_vel_z__011,SKILL6;	// base velocity

define	_off_x__011,SKILL4;	// starting offset
define	_off_y__011,SKILL5;	// starting offset
define	_off_z__011,SKILL6;	// starting offset

define	_elapse_time__011,SKILL21;	// elapsed time since last particle


// Local temp values
var	particle00_tmpStart[3];
var	particle00_tmpEnd[3];

// Desc: create a random vector
function Particle00_Randomize(&vec, range)
{
	vec[0] = random(1) - 0.5;
	vec[1] = random(1) - 0.5;
	vec[2] = random(1) - 0.5;
	vec_normalize(vec,random(range));
}


//----------------------------------------------------------------------
// Draw Line::

// Desc: DrawLine effect helper function
// Called from: Particle00_DrawLine,
function Particle00_DrawLine_Func()
{
// section: Draw line effect.
//entry: line red value
// id: 1
	my.red = 255;
//entry: line green value
// id: 2
	my.green = 0;
//entry: line blue value
// id: 3
	my.blue = 0;
//entry: line size
// id: 4
	my.size = 1;
//entry: line duration (ticks)
// id: 5
	my.lifespan = 1;
	my.streak = on;
	my.function = NULL;
}

// Desc: draw line from start to end
// Note! Only works in Commercial edition or higher (streak effect).
function Particle00_DrawLine(&start,&end)
{
	vec_diff(temp,end,start);
	effect(Particle00_DrawLine_Func,1,start,temp);
}

//----------------------------------------------------------------------
// Draw OBB::
// Draw an OBB around the entity that calls this function
// NOTE: MY must be a valid entity!
function	Particle00_DrawOBB()
{
	// draw OBB upper
	vec_set(particle00_tmpStart,vector(my.max_x,my.max_y,my.max_z));
	vec_rotate(particle00_tmpStart,my.pan);
	vec_add(particle00_tmpStart,my.x);
	vec_set(particle00_tmpEnd,vector(my.max_x,my.min_y,my.max_z));
	vec_rotate(particle00_tmpEnd,my.pan);
	vec_add(particle00_tmpEnd,my.x);
	Particle00_DrawLine(particle00_tmpStart,particle00_tmpEnd);

	vec_set(particle00_tmpStart,vector(my.min_x,my.min_y,my.max_z));
	vec_rotate(particle00_tmpStart,my.pan);
	vec_add(particle00_tmpStart,my.x);
	Particle00_DrawLine(particle00_tmpStart,particle00_tmpEnd);

	vec_set(particle00_tmpEnd,vector(my.min_x,my.max_y,my.max_z));
	vec_rotate(particle00_tmpEnd,my.pan);
	vec_add(particle00_tmpEnd,my.x);
	Particle00_DrawLine(particle00_tmpStart,particle00_tmpEnd);

	vec_set(particle00_tmpStart,vector(my.max_x,my.max_y,my.max_z));
	vec_rotate(particle00_tmpStart,my.pan);
	vec_add(particle00_tmpStart,my.x);
	Particle00_DrawLine(particle00_tmpStart,particle00_tmpEnd);

	// draw OBB lower
	vec_set(particle00_tmpStart,vector(my.max_x,my.max_y,my.min_z));
	vec_rotate(particle00_tmpStart,my.pan);
	vec_add(particle00_tmpStart,my.x);
	vec_set(particle00_tmpEnd,vector(my.max_x,my.min_y,my.min_z));
	vec_rotate(particle00_tmpEnd,my.pan);
	vec_add(particle00_tmpEnd,my.x);
	Particle00_DrawLine(particle00_tmpStart,particle00_tmpEnd);

	vec_set(particle00_tmpStart,vector(my.min_x,my.min_y,my.min_z));
	vec_rotate(particle00_tmpStart,my.pan);
	vec_add(particle00_tmpStart,my.x);
	Particle00_DrawLine(particle00_tmpStart,particle00_tmpEnd);

	vec_set(particle00_tmpEnd,vector(my.min_x,my.max_y,my.min_z));
	vec_rotate(particle00_tmpEnd,my.pan);
	vec_add(particle00_tmpEnd,my.x);
	Particle00_DrawLine(particle00_tmpStart,particle00_tmpEnd);

	vec_set(particle00_tmpStart,vector(my.max_x,my.max_y,my.min_z));
	vec_rotate(particle00_tmpStart,my.pan);
	vec_add(particle00_tmpStart,my.x);
	Particle00_DrawLine(particle00_tmpStart,particle00_tmpEnd);

	// draw OBB sides
	vec_set(particle00_tmpStart,vector(my.max_x,my.max_y,my.max_z));
	vec_rotate(particle00_tmpStart,my.pan);
	vec_add(particle00_tmpStart,my.x);
	vec_set(particle00_tmpEnd,vector(my.max_x,my.max_y,my.min_z));
	vec_rotate(particle00_tmpEnd,my.pan);
	vec_add(particle00_tmpEnd,my.x);
	Particle00_DrawLine(particle00_tmpStart,particle00_tmpEnd);

	vec_set(particle00_tmpStart,vector(my.min_x,my.min_y,my.max_z));
	vec_rotate(particle00_tmpStart,my.pan);
	vec_add(particle00_tmpStart,my.x);
	vec_set(particle00_tmpEnd,vector(my.min_x,my.min_y,my.min_z));
	vec_rotate(particle00_tmpEnd,my.pan);
	vec_add(particle00_tmpEnd,my.x);
	Particle00_DrawLine(particle00_tmpStart,particle00_tmpEnd);

	vec_set(particle00_tmpStart,vector(my.min_x,my.max_y,my.max_z));
	vec_rotate(particle00_tmpStart,my.pan);
	vec_add(particle00_tmpStart,my.x);
	vec_set(particle00_tmpEnd,vector(my.min_x,my.max_y,my.min_z));
	vec_rotate(particle00_tmpEnd,my.pan);
	vec_add(particle00_tmpEnd,my.x);
	Particle00_DrawLine(particle00_tmpStart,particle00_tmpEnd);

	vec_set(particle00_tmpStart,vector(my.max_x,my.min_y,my.max_z));
	vec_rotate(particle00_tmpStart,my.pan);
	vec_add(particle00_tmpStart,my.x);
	vec_set(particle00_tmpEnd,vector(my.max_x,my.min_y,my.min_z));
	vec_rotate(particle00_tmpEnd,my.pan);
	vec_add(particle00_tmpEnd,my.x);
	Particle00_DrawLine(particle00_tmpStart,particle00_tmpEnd);

}





//----------------------------------------------------------------------
// section: Smoke effect

// entry: Smoke particle bitmap
bmap	particle00_smoke_bmp = <smkpart.tga>;// smoke particle

// Desc: Update smoke effect
function	Particle00_Smoke_Update()
{
	my.alpha -= (2.35 * time_step);
	if(my.alpha < 1) { my.lifespan = 0; }  // remove
	my.SIZE += (0.75 * time_step);
}

// Desc: smaller, more transparent, less movement
// Use: effect(Particle00_Smoke_FX2,5,start_pos_vec,base_speed_vec)
function Particle00_Smoke_FX2()
{
	my.vel_x += random(0.5) - 0.25;
	my.vel_y += random(0.5) - 0.25;
	my.vel_z += random(0.5) - 0.25;
	my.alpha = 35;
	my.bmap = particle00_smoke_bmp; // smoke part

	my.size = 0.125;
	my.lifespan = 6400;
	my.flare = off;
	my.transparent = on;
	my.move = on;
	my.function = Particle00_Smoke_Update;	// smoke update funtion
}

// Desc: handle smoke particles
// Use: effect(Particle00_Smoke_FX,5,start_pos_vec,base_speed_vec)
function Particle00_Smoke_FX()
{
	my.vel_x += random(1) - 0.5;
	my.vel_y += random(1) - 0.5;
	my.vel_z += random(2)-1;
	my.alpha = 64;
	my.bmap = particle00_smoke_bmp; // smoke part

	my.size = 1;
	my.lifespan = 6400;
	my.flare = off;
	my.transparent = on;
	my.move = on;
	my.function = Particle00_Smoke_Update;	// smoke update funtion
}



// Desc: smoke emiter with trigger
function	Particle00_Smoker_Triggered()
{
	while(1)
	{
		if(Trigger00_Check(my._use_trigger__011,trigger00_bitlist) == 1)
		{
			my._elapse_time__011 += time_step;
			temp = int(my._elapse_time__011 / my._rate__011);
			if(temp > 0)
			{
	 			effect(Particle00_Smoke_FX,temp,my.x,my._vel_x__011);
			}
			my._elapse_time__011 %= my._rate__011;
		}
		wait(1);
	}
}

// Desc: smoke emiter without trigger
function	Particle00_Smoker_Always()
{
	while(1)
	{
		// calculate number of particles needed this frame
		my._elapse_time__011 += time_step;
		temp = int(my._elapse_time__011 / my._rate__011);
		if(temp > 0)
		{
	 		effect(Particle00_Smoke_FX,temp,my.x,my._vel_x__011);
		}
		my._elapse_time__011 %= my._rate__011;
		wait(1);
	}
}

// Desc: a smoke emiter
//
// uses: _rate__011, _use_trigger__011,
// uses: _vel_x__011, _vel_y__011, _vel_z__011
action	Particle00_Smoker
{

	if(my._rate__011 <= 0) { my._rate__011 = 1; } // default value

	if(my._use_trigger__011 <= 0)
	{
		Particle00_Smoker_Always();	// no need to check for a trigger each frame
	}

	Particle00_Smoker_Triggered();	// check for a trigger each frame


}

//----------------------------------------------------------------------
// section: Floating particles
// Floter::  A particle that floats up

// entry: Float particle bitmap
bmap	particle00_float_bmp = <spkpart.bmp>;


// Desc: Update smoke effect
function	Particle00_Float_Update()
{
	if(my.lifespan < 25)
	{
		my.alpha = my.lifespan;
	}
}


// Desc: handle float particles
// Use: effect(Particle00_Float_FX,number,start_pos_vec,range_offset_vec)
//
function Particle00_Float_FX()
{
	// set init offset
	my.x += random(my.vel_x) - (0.5*my.vel_x);
	my.y += random(my.vel_y) - (0.5*my.vel_y);
	my.z += random(my.vel_z) - (0.5*my.vel_z);

	// set floating speed
	my.vel_x = 0;
	my.vel_y = 0;
// entry: floating speed
// id: 10
	my.vel_z = 2;

// entry: alpha value
// id: 11
	my.alpha = 25;
	my.bmap = particle00_float_bmp;

// entry: scale
// id: 12
	my.size = 5;
// entry: lifespan (in ticks)
// id: 13
	my.lifespan = 32;
	my.flare = off;
	my.transparent = on;
	my.move = on;


	my.function = Particle00_Float_Update;
}


// Desc: floating particle emiter with trigger
function	Particle00_Float_Triggered()
{
	while(1)
	{
		if(Trigger00_Check(my._use_trigger__011,trigger00_bitlist) == 1)
		{
			my._elapse_time__011 += time_step;
			temp = int(my._elapse_time__011 / my._rate__011);
			if(temp > 0)
			{
	 			effect(Particle00_Float_FX,temp,my.x,my._off_x__011);
			}
			my._elapse_time__011 %= my._rate__011;
		}
		wait(1);
	}
}

// Desc: floating particle emiter without trigger
function	Particle00_Float_Always()
{
	while(1)
	{
		// calculate number of particles needed this frame
		my._elapse_time__011 += time_step;
		temp = int(my._elapse_time__011 / my._rate__011);
		if(temp > 0)
		{
	 		effect(Particle00_Float_FX,temp,my.x,my._off_x__011);
		}
		my._elapse_time__011 %= my._rate__011;
		wait(1);
	}
}

// Desc: a floating particle emiter
//
// uses: _rate__011, _use_trigger__011,
// uses: _off_x__011, _off_y__011, _off_z__011
action	Particle00_Float
{

	if(my._rate__011 <= 0) { my._rate__011 = 1; } // default value

	if(my._use_trigger__011 <= 0)
	{
		Particle00_Float_Always();	// no need to check for a trigger each frame
	}

	Particle00_Float_Triggered();	// check for a trigger each frame


}

endif;